<?php
/**
 * @file
 *   Page and other call backs for WebAPI site manager
 */

/**
 * Form API cllback
 */
function tycoon_webapi_site_manager_key_details($form, &$form_state, $kid) {
  $form['#prefix'] = '<div id="tycoon_site_manager">';
  $form['#suffix'] = '</div>';
  
  $form['title'] = array(
    '#type' => 'markup',
    '#markup' => '<h4>' . t('Associated Sites') . '</h4>',
  );
  $ajax = array(
    '#ajax' => array(
      'wrapper' => 'tycoon_site_manager',
      'callback' => 'tycoon_webapi_site_ajax',
      'method' => 'replace',
    ),
  );
  $sites = db_select('tycoon_webapi_sites', 's')
    ->condition('kid', $kid)
    ->fields('s')
    ->orderBy('name', 'asc')
    ->execute();
  $form['sites'] = array(
    '#type' => 'container',
    '#tree' => TRUE,
    '#theme' => 'tycoon_webapi_site_manager_key_details_table'
  );
  while ($site = $sites->fetch()) {
    $form['sites'][$site->sid]['site'] = array('#type' => '#value',  '#value' => $site);
    $form['sites'][$site->sid]['action'] = array(
      '#type' => 'submit',
      '#value' => t('Delete Site'),
      '#name' => 'site_' . $site->sid,
      '#submit' => array('tycoon_webapi_site_manager_key_details_delete')
    ) + $ajax;
  }
  $form['sites']['add']['#weight'] = 50;
  $form['sites']['add']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Site name'),
  );
  $form['sites']['add']['ip'] = array(
    '#type' => 'textfield',
    '#title' => t('IP Address'),
    '#description' => t('Must be in IPV4 format ###.###.###.###.'),
    '#element_validate' => array('ip_form_validate'),
  );
  $form['sites']['add']['kid'] = array(
    '#type' => 'value',
    '#value' => $kid,
  );
  $form['sites']['add']['action'] = array(
    '#type' => 'submit',
    '#value' => t('Add Site'),
  ) + $ajax;
  
  $form['#kid'] = $kid;
  
  return $form;
}

/**
 * Form API submit callback
 */
function _tycoon_webapi_site_manager_key_details_submit($form, &$form_state) {
  $add = $form_state['values']['sites']['add'];
  unset($add['action']);
  $sid = db_insert('tycoon_webapi_sites')->fields($add)->execute();
  if ($sid) {
    drupal_set_message(t('Site added to API Key sites.'));
  }
  unset($form_state['input']);
  $form_state['rebuild'] = TRUE;
}

/**
 * Form API submit callback
 */
function _tycoon_webapi_site_manager_key_details_delete($form, &$form_state) {
  $match = array();
  preg_match('/^site\_([0-9]+)$/', $form_state['clicked_button']['#name'], $match);
  if (isset($match[1])) {
    db_delete('tycoon_webapi_sites')->condition('sid', $match[1])->execute();
    drupal_set_message(t('Site deleted.'));
  }
  $form_state['rebuild'] = TRUE;
}
 
/**
 * Theme function for tycoon_webapi_site_manager_key_details
 */
function theme_tycoon_webapi_site_manager_key_details_table($vars) {
  $form = $vars['form'];
  foreach(element_children($form) as $key) {
    if ($key == 'add') {
      unset($form[$key]['name']['#title'], $form[$key]['ip']['#title']);
      $add_row = array();
      $add_row[] = drupal_render($form[$key]['name']);
      $add_row[] = drupal_render($form[$key]['ip']);
      $add_row[] = drupal_render($form[$key]['action']);
    }
    else {
      $site = $form[$key]['site']['#value'];
      $row = array();
      $row[] = $site->name;
      $row[] = $site->ip;
      $row[] = drupal_render($form[$key]['action']);
    
      $tvars['rows'][] = $row;
    }
  }
  if (isset($add_row)) {
    $tvars['rows'][] = $add_row;
  }
  $tvars['header'] = array(
    t('Site name'),
    t('IP Address'),
    t('Add/Delete'),
  );
  return theme('table', $tvars) . drupal_render_children($form);
}
 
/**
 * Form API cllback
 */
function tycoon_webapi_site_manager_key_details_strict($form, &$form_state, $kid) {
  $strict = db_select('tycoon_webapi_keys_strict', 's')
    ->fields('s')
    ->condition('kid', $kid)
    ->execute()
    ->fetch();
  if (!$strict) {
    $strict = new stdClass();
    $strict->log = 0;
    $strict->strict = 0;
  }
  $form['#prefix'] = '<div id="tycoon_site_manager_strict">';
  $form['#suffix'] = '</div>';
  $form['kid'] = array(
    '#type' => 'value',
    '#value' => $kid,
  );
  $form['strict'] = array(
    '#type' => 'checkbox',
    '#title' => t('Only allow calls from the above IP Addresses.'),
    '#default_value' => $strict->strict,
    '#ajax' => array(
      'wrapper' => 'tycoon_site_manager_strict',
      'callback' => 'tycoon_webapi_site_ajax',
      'method' => 'replace',
      'progress' => array(
        'message' => '',
      ),
      'trigger_as' => array(
        'name' => 'op',
      ),
    ),
  );
  $form['log'] = array(
    '#type' => 'checkbox',
    '#title' => t('Log site actions for this key.'),
    '#default_value' => $strict->log,
    '#access' => !variable_get('tycoon_webapi_site_log_all', 0),
    '#ajax' => array(
      'wrapper' => 'tycoon_site_manager_strict',
      'callback' => 'tycoon_webapi_site_ajax',
      'method' => 'replace',
      'progress' => array(
        'message' => '',
      ),
      'trigger_as' => array(
        'name' => 'op',
      ),
    ),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#states' => array(
      //This basically keeps the button hidden any time JS is enabled.
      //Which since we are using an #ajax call back on the checkbox is 
      //exactly what we want.
      'visible' => array(
        ':input[name="op"]' => array('value' => md5(1)),
      ),
    ),
  );
  
  return $form;
}

/**
 * Form API submit callback
 */
function _tycoon_webapi_site_manager_key_details_strict_submit($form, &$form_state) {
  $vals = $form_state['values'];
  $key->kid = $vals['kid'];
  $key->strict = $vals['strict'];
  $key->log = $vals['log'];
  db_delete('tycoon_webapi_keys_strict')
    ->condition('kid', $vals['kid'])
    ->execute();
  db_insert('tycoon_webapi_keys_strict')
    ->fields((array)$key)
    ->execute();
  unset($form_state['input']);
  $form_state['rebuild'] = TRUE;
}

/**
 * Builds the table of logs for a specific key or all of no $key is specified
 * 
 * @param $kid
 *   The key ID from which to build the table of data.
 * @param $rows
 *   The number of rows to build per page
 * @param $element
 *   The pager element to use
 * 
 * @return
 *    A Fully built table of data
 */
function tycoon_webapi_site_manager_tycoon_webapi_log_table($kid = NULL, $rows = 10, $element = 0) {
  $page = pager_find_page($element) * $rows;
  $vars['header'][] = array(
    'data' => t('Log ID'),
    'field' => 'lid',
    'sort' => 'asc',
  );
  if (!$kid) {
    $vars['header'][] = array(
      'data' => t('API Key'),
      'field' => 'api_key',
    );
  }
  $vars['header'][] = array(
    'data' => t('Message'),
  );
  $vars['header'][] = array(
    'data' => t('Date'),
    'field' => 'created',
  );
  $vars['header'][] = array(
    'data' => t('Details'),
  );
  $sort = tablesort_get_sort($vars['header']);
  $order = tablesort_get_order($vars['header']);
  
  $logs_q = db_select('tycoon_webapi_sites_log', 'l')
    ->orderBy($order['sql'], $sort)
    ->orderBy('created', 'asc')
    ->fields('l');
  if (!$kid) {
    $alias = $logs_q->join('tycoon_webapi_keys', 'k', 'k.kid = l.kid');
    $logs_q->fields($alias);
  }
  else {
    $logs_q->condition('kid', $kid);
  }
  $count_q = $logs_q->countQuery();
  $logs = $logs_q->range($page, $rows)->execute();
  $count = $count_q->execute()->fetchField();
  while ($log = $logs->fetch()) {
    $opts = array();
    $row = array();
    $row[] = $log->lid;
    if (!$kid) {
      $row[] = $log->api_key;
    }
    else {
      $opts['query']['return'] = 'key_details';
    }
    $row[] = t($log->message, unserialize($log->args));
    $row[] = format_date($log->created);
    $row[] = l('Details', 'admin/config/tycoon/webapi/log_details/' . $log->lid, $opts);
    $vars['rows'][] = $row;
  }
  
  $vars['caption'] = t('API Logs');
  $vars['empty'] = t('There are no logs to display');
  pager_default_initialize($count, $rows, $element);
  return theme('table', $vars) . theme('pager', array('element' => $element));
}

/**
 * Log Details for sites
 */
function tycoon_webapi_site_manager_log_details($lid = 0) {
  $logs_q = db_select('tycoon_webapi_sites_log', 'l')
    ->fields('l')
    ->condition('lid', $lid);
  $alias = $logs_q->join('tycoon_webapi_keys', 'k', 'k.kid = l.kid');
  $log = $logs_q->fields($alias)->execute()->fetch();
  if (!$log) {
    return drupal_not_found();
  }
  $message = t($log->message, unserialize($log->args));
  
  drupal_set_title($message);
  
  $rows[] = array(
    array(
      'data' => t('Log ID'),
      'class' => 'active',
    ),
    $log->lid,    
  );
  $rows[] = array(
    array(
      'data' => t('API Key'),
      'class' => 'active',
    ),
    l($log->api_key, 'admin/config/tycoon/webapi/key_details/' . $log->kid),    
  );
  $rows[] = array(
    array(
      'data' => t('Command'),
      'class' => 'active',
    ),
    $log->command,    
  );
  $rows[] = array(
    array(
      'data' => t('Date'),
      'class' => 'active',
    ),
    format_date($log->created),    
  );
  $rows[] = array(
    array(
      'data' => t('Message'),
      'class' => 'active',
    ),
    $message,    
  );
  $data = unserialize($log->data);
  if (module_exists('devel') && merits_krumo($data)) {
    $data_out = krumo_ob($data);
  }
  else {
    ob_start();
    var_dump($data);
    $data_out = ob_get_clean();
 
  }
  
  $rows[] = array(
    array(
      'data' => t('Data'),
      'class' => 'active',
    ),
    $data_out,    
  );
  
  $vars['rows'] = $rows;
  if ($_GET['return'] == 'key_details') {
    $vars['caption'] = l('«Return to Key Details', 'admin/config/tycoon/webapi/key_details/' . $log->kid);
  }
  return theme('table', $vars);
}
